clear; clc; tic;

%------------------------------------------------------------------------
%                       Simulation Setup 
%------------------------------------------------------------------------
S = 100000;  B = 999;  N_Size = 0.1;  cv = norminv(1-N_Size/2);

IQR_G   = norminv(1 - 0.25) - norminv(0.25);

DGP_1   = kron(ones(4,1),[0.025;0.2]);  

DGP_2   = [kron([0.25;0.5],ones(2,1));kron([0.25;0.5],ones(2,1))]; 

DGP_3   = kron([100;1000],ones(4,1));

DGP     = [DGP_1, DGP_2, DGP_3];

Emp_Rej = zeros(8,7);

SD_est_prop1 = zeros(8,7);

SD_est_prop2 = zeros(8,7);

for m   = 1:8
    
R2 = DGP(m,1); rho = DGP(m,2); N = DGP(m,3); 
    
pi = (R2/(1-R2))^.5;  Asy_SD = 1/pi;
    
Em_Rej_B  = zeros(S,1);      Em_Rej_PB = zeros(S,1);  

Em_Rej_P1 = zeros(S,1);      Em_Rej_P2 = zeros(S,1); 

parfor s = 1:S

%-------------------------------------------------------------------------- 
%                     Generate Data (beta = 0)
%--------------------------------------------------------------------------   
Sig  = [1 rho 0; rho 1 0; 0 0 1];
 
D    = mvnrnd(zeros(3,1),Sig,N); 

Z    = D(:,3);

X    = Z*pi + D(:,2);

Y    = D(:,1);
%-------------------------------------------------------------------------- 
%  Calculate the LIML estimator and the Student-t Statistic (H0: beta = 0)
%--------------------------------------------------------------------------       
est    = (Z'*Y)/(Z'*X);
    
est_sd = sqrt(mean((Y - X*est).^2)*(Z'*Z/N)/(X'*Z/N)^2);
    
Stud_t = sqrt(N)*est/est_sd;
%-------------------------------------------------------------------------- 
%                  Getting the bootstrap sample
%--------------------------------------------------------------------------  
[~,b_id]  = bootstrp(B,[],Y); est_B = zeros(B,1); est_B_sd = zeros(B,1);
    
for b = 1:B

    Y_b = Y(b_id(:,b),:);  X_b = X(b_id(:,b),:);  Z_b = Z(b_id(:,b),:); 
        
    est_B(b,1)    = (Z_b'*Y_b)/(Z_b'*X_b);
        
    est_B_sd(b,1) = sqrt(mean((Y_b - X_b*est_B(b,1)).^2)*(Z_b'*Z_b/N)/(X_b'*Z_b/N)^2);
    
end

    D_est = est_B - est;
    
    N_est = sqrt(N)*D_est;
    
    T_est = N_est./est_B_sd;
%-------------------------------------------------------------------------- 
%             Inference based on the bootstrap estimator
%--------------------------------------------------------------------------   
    S_B  = ( mean(N_est.^2) - mean(N_est)^2 )^.5;
    
    t_B  = sqrt(N)*est/S_B;
    
    Em_Rej_B(s,:) = (abs(t_B) > cv);
%-------------------------------------------------------------------------- 
%       Inference based on Machado and Parente (2005)'s IQR estimator
%--------------------------------------------------------------------------
    S_PB = iqr(N_est)/IQR_G;
    
    t_PB = sqrt(N)*est/S_PB;
    
    Em_Rej_PB(s,:) = (abs(t_PB) > cv);
%-------------------------------------------------------------------------- 
%        Inference based on the bootstrap percentile
%--------------------------------------------------------------------------    
    b_q11 = quantile(D_est,N_Size/2);  
    
    b_q12 = quantile(D_est,1-N_Size/2); 

    Em_Rej_P1(s,:) = (est < b_q11 || est > b_q12);
%-------------------------------------------------------------------------- 
%        Inference based on the bootstrap percentile-t
%--------------------------------------------------------------------------    
    b_q21 = quantile(T_est,N_Size/2);  
    
    b_q22 = quantile(T_est,1-N_Size/2); 

    Em_Rej_P2(s,:) = (Stud_t < b_q21 || Stud_t > b_q22); 
%-------------------------------------------------------------------------- 
%        Save the SD estimators
%--------------------------------------------------------------------------   

    SD_est(s,:) = [S_B, S_PB];

end
%-------------------------------------------------------------------------- 
%        Creat the table for the empirical rejection probabilties
%--------------------------------------------------------------------------

Emp_Rej(m,:)      = [N, R2, rho, mean(Em_Rej_B), mean(Em_Rej_PB), mean(Em_Rej_P1), mean(Em_Rej_P2)];

%-------------------------------------------------------------------------- 
%        Creat the table for the properties of the sd estimators
%--------------------------------------------------------------------------  
SD_est_ratio      = SD_est./(Asy_SD*ones(S,2));

Mean_SD_est_ratio = mean(SD_est_ratio);

Med_SD_est_ratio  = median(SD_est_ratio);

SD_SD_est_ratio   = sqrt(mean(SD_est_ratio.^2) - mean(SD_est_ratio).^2);

IQR_SD_est_ratio  = iqr(SD_est_ratio);

SD_est_table0     = [Mean_SD_est_ratio', SD_SD_est_ratio', Med_SD_est_ratio', IQR_SD_est_ratio'];

SD_est_prop1(m,:) = [N, R2, rho, SD_est_table0(1,:)];

SD_est_prop2(m,:) = [N, R2, rho, SD_est_table0(2,:)];
end
%-------------------------------------------------------------------------- 
%             Save the Results
%--------------------------------------------------------------------------
colNames1         = {'Sample Size', 'R2', 'Rho', 'B_SD', 'PB_SD(IQR)', 'Boot_P', 'Boot_PT'};

Emp_Rej_table     = array2table(Emp_Rej,'VariableNames',colNames1);
                                                   
colNames2         = {'Sample Size', 'R2', 'Rho', 'Mean', 'S.d.', 'Median', 'IQR'};

SD_est_table1     =  array2table(SD_est_prop1,'VariableNames',colNames2);

SD_est_table2     =  array2table(SD_est_prop2,'VariableNames',colNames2);

head     =  'Simulation_Tables';

save(sprintf(head),'Emp_Rej_table', 'SD_est_table1', 'SD_est_table2');

toc
